@@ -59,10 +59,10 @@ import java.io.FileReader;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 
-import org.eclipse.jgit.JGitText;
 import org.eclipse.jgit.errors.ConfigInvalidException;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.errors.MissingObjectException;
+import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.lib.AnyObjectId;
 import org.eclipse.jgit.lib.CommitBuilder;
 import org.eclipse.jgit.lib.Constants;
@@ -370,9 +370,8 @@ public class T0003_BasicTest extends SampleDataRepositoryTestCase {
 		try {
 			new FileRepository(db.getDirectory());
 			fail("incorrectly opened a bad repository");
-		} catch (IOException ioe) {
-			assertTrue(ioe.getMessage().indexOf("format") > 0);
-			assertTrue(ioe.getMessage().indexOf(badvers) > 0);
+		} catch (IllegalArgumentException ioe) {
+			assertNotNull(ioe.getMessage());
 		}
 	}
 
