@@ -54,7 +54,8 @@ import org.eclipse.jgit.util.FS;
 /**
  * Offers a "GitPorcelain"-like API to interact with a git repository.
  * <p>
- * The GitPorcelain commands are described in the <a href="http://www.kernel.org/pub/software/scm/git/docs/git.html#_high_level_commands_porcelain"
+ * The GitPorcelain commands are described in the <a href=
+ * "http://www.kernel.org/pub/software/scm/git/docs/git.html#_high_level_commands_porcelain"
  * >Git Documentation</a>.
  * <p>
  * This class only offers methods to construct so-called command classes. Each
@@ -109,9 +110,7 @@ public class Git {
 		RepositoryCache.FileKey key;
 
 		key = RepositoryCache.FileKey.lenient(dir, fs);
-		return wrap(new RepositoryBuilder()
-				.setFS(fs)
-				.setGitDir(key.getFile())
+		return wrap(new RepositoryBuilder().setFS(fs).setGitDir(key.getFile())
 				.setMustExist(true).build());
 	}
 
@@ -266,8 +265,8 @@ public class Git {
 	 * @see <a
 	 *      href="http://www.kernel.org/pub/software/scm/git/docs/git-add.html"
 	 *      >Git documentation about Add</a>
-	 * @return a {@link AddCommand} used to collect all optional parameters
-	 *         and to finally execute the {@code Add} command
+	 * @return a {@link AddCommand} used to collect all optional parameters and
+	 *         to finally execute the {@code Add} command
 	 */
 	public AddCommand add() {
 		return new AddCommand(repo);
@@ -279,8 +278,8 @@ public class Git {
 	 * @see <a
 	 *      href="http://www.kernel.org/pub/software/scm/git/docs/git-tag.html"
 	 *      >Git documentation about Tag</a>
-	 * @return a {@link TagCommand} used to collect all optional parameters
-	 *         and to finally execute the {@code Tag} command
+	 * @return a {@link TagCommand} used to collect all optional parameters and
+	 *         to finally execute the {@code Tag} command
 	 */
 	public TagCommand tag() {
 		return new TagCommand(repo);
@@ -331,8 +330,8 @@ public class Git {
 	 * @see <a
 	 *      href="http://www.kernel.org/pub/software/scm/git/docs/git-revert.html"
 	 *      >Git documentation about reverting changes</a>
-	 * @return a {@link RevertCommand} used to collect all optional
-	 *         parameters and to finally execute the {@code cherry-pick} command
+	 * @return a {@link RevertCommand} used to collect all optional parameters
+	 *         and to finally execute the {@code cherry-pick} command
 	 */
 	public RevertCommand revert() {
 		return new RevertCommand(repo);
@@ -573,6 +572,51 @@ public class Git {
 		return new StashListCommand(repo);
 	}
 
+	/**
+	 * Returns a command object used to create a stashed commit
+	 *
+	 * @return a {@link StashCreateCommand}
+	 * @since 2.0
+	 */
+	public StashCreateCommand stashCreate() {
+		return new StashCreateCommand(repo);
+	}
+
+	/**
+	 * Returns a command object used to apply a stashed commit
+	 *
+	 * @return a {@link StashApplyCommand}
+	 * @since 2.0
+	 */
+	public StashApplyCommand stashApply() {
+		return new StashApplyCommand(repo);
+	}
+
+	/**
+	 * Returns a command object used to drop a stashed commit
+	 *
+	 * @return a {@link StashDropCommand}
+	 * @since 2.0
+	 */
+	public StashDropCommand stashDrop() {
+		return new StashDropCommand(repo);
+	}
+
+	/**
+	 * Returns a command object to execute a {@code apply} command
+	 *
+	 * @see <a
+	 *      href="http://www.kernel.org/pub/software/scm/git/docs/git-apply.html"
+	 *      >Git documentation about apply</a>
+	 *
+	 * @return a {@link ApplyCommand} used to collect all optional parameters
+	 *         and to finally execute the {@code apply} command
+	 * @since 2.0
+	 */
+	public ApplyCommand apply() {
+		return new ApplyCommand(repo);
+	}
+
 	/**
 	 * @return the git repository this class is interacting with
 	 */
