@@ -62,7 +62,6 @@ import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 
-import org.eclipse.jgit.JGitText;
 import org.eclipse.jgit.dircache.DirCache;
 import org.eclipse.jgit.errors.AmbiguousObjectException;
 import org.eclipse.jgit.errors.CorruptObjectException;
@@ -74,6 +73,7 @@ import org.eclipse.jgit.events.IndexChangedEvent;
 import org.eclipse.jgit.events.IndexChangedListener;
 import org.eclipse.jgit.events.ListenerList;
 import org.eclipse.jgit.events.RepositoryEvent;
+import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.revwalk.RevBlob;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevObject;
@@ -871,7 +871,7 @@ public abstract class Repository {
 	 */
 	public DirCache readDirCache() throws NoWorkTreeException,
 			CorruptObjectException, IOException {
-		return DirCache.read(getIndexFile(), getFS());
+		return DirCache.read(this);
 	}
 
 	/**
@@ -903,7 +903,7 @@ public abstract class Repository {
 				notifyIndexChanged();
 			}
 		};
-		return DirCache.lock(getIndexFile(), getFS(), l);
+		return DirCache.lock(this, l);
 	}
 
 	static byte[] gitInternalSlash(byte[] bytes) {
@@ -1247,6 +1247,39 @@ public abstract class Repository {
 		writeHeadsFile(heads, Constants.CHERRY_PICK_HEAD);
 	}
 
+	/**
+	 * Write original HEAD commit into $GIT_DIR/ORIG_HEAD.
+	 *
+	 * @param head
+	 *            an object id of the original HEAD commit or <code>null</code>
+	 *            to delete the file
+	 * @throws IOException
+	 */
+	public void writeOrigHead(ObjectId head) throws IOException {
+		List<ObjectId> heads = head != null ? Collections.singletonList(head)
+				: null;
+		writeHeadsFile(heads, Constants.ORIG_HEAD);
+	}
+
+	/**
+	 * Return the information stored in the file $GIT_DIR/ORIG_HEAD.
+	 *
+	 * @return object id from ORIG_HEAD file or {@code null} if this file
+	 *         doesn't exist. Also if the file exists but is empty {@code null}
+	 *         will be returned
+	 * @throws IOException
+	 * @throws NoWorkTreeException
+	 *             if this is bare, which implies it has no working directory.
+	 *             See {@link #isBare()}.
+	 */
+	public ObjectId readOrigHead() throws IOException, NoWorkTreeException {
+		if (isBare() || getDirectory() == null)
+			throw new NoWorkTreeException();
+
+		byte[] raw = readGitDirectoryFile(Constants.ORIG_HEAD);
+		return raw != null ? ObjectId.fromString(raw, 0) : null;
+	}
+
 	/**
 	 * Read a file from the git directory.
 	 *
