@@ -65,10 +65,10 @@ import java.util.concurrent.ExecutionException;
 import java.util.zip.DataFormatException;
 import java.util.zip.Inflater;
 
-import org.eclipse.jgit.JGitText;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.errors.StoredObjectRepresentationNotAvailableException;
+import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.lib.AbbreviatedObjectId;
 import org.eclipse.jgit.lib.AnyObjectId;
 import org.eclipse.jgit.lib.AsyncObjectLoaderQueue;
@@ -541,7 +541,6 @@ final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		}
 	}
 
-	@SuppressWarnings("unchecked")
 	public Collection<CachedPack> getCachedPacks() throws IOException {
 		DfsPackFile[] packList = db.getPacks();
 		List<CachedPack> cached = new ArrayList<CachedPack>(packList.length);
