@@ -55,11 +55,11 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.eclipse.jgit.JGitText;
 import org.eclipse.jgit.errors.CorruptObjectException;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.errors.PackProtocolException;
+import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.NullProgressMonitor;
 import org.eclipse.jgit.lib.ObjectId;
@@ -118,6 +118,44 @@ public class UploadPack {
 		ANY;
 	}
 
+	/** Data in the first line of a request, the line itself plus options. */
+	public static class FirstLine {
+		private final String line;
+		private final Set<String> options;
+
+		/**
+		 * Parse the first line of a receive-pack request.
+		 *
+		 * @param line
+		 *            line from the client.
+		 */
+		public FirstLine(String line) {
+			if (line.length() > 45) {
+				final HashSet<String> opts = new HashSet<String>();
+				String opt = line.substring(45);
+				if (opt.startsWith(" "))
+					opt = opt.substring(1);
+				for (String c : opt.split(" "))
+					opts.add(c);
+				this.line = line.substring(0, 45);
+				this.options = Collections.unmodifiableSet(opts);
+			} else {
+				this.line = line;
+				this.options = Collections.emptySet();
+			}
+		}
+
+		/** @return non-capabilities part of the line. */
+		public String getLine() {
+			return line;
+		}
+
+		/** @return options parsed from the line. */
+		public Set<String> getOptions() {
+			return options;
+		}
+	}
+
 	/** Database we read the objects from. */
 	private final Repository db;
 
@@ -157,14 +195,17 @@ public class UploadPack {
 	/** The refs we advertised as existing at the start of the connection. */
 	private Map<String, Ref> refs;
 
+	/** Hook used while advertising the refs to the client. */
+	private AdvertiseRefsHook advertiseRefsHook = AdvertiseRefsHook.DEFAULT;
+
 	/** Filter used while advertising the refs to the client. */
-	private RefFilter refFilter;
+	private RefFilter refFilter = RefFilter.DEFAULT;
 
 	/** Hook handling the various upload phases. */
 	private PreUploadHook preUploadHook = PreUploadHook.NULL;
 
 	/** Capabilities requested by the client. */
-	private final Set<String> options = new HashSet<String>();
+	private Set<String> options;
 
 	/** Raw ObjectIds the client has asked for, before validating them. */
 	private final Set<ObjectId> wantIds = new HashSet<ObjectId>();
@@ -241,7 +282,6 @@ public class UploadPack {
 		SAVE.add(PEER_HAS);
 		SAVE.add(COMMON);
 		SAVE.add(SATISFIED);
-		refFilter = RefFilter.DEFAULT;
 	}
 
 	/** @return the repository this upload is reading from. */
@@ -254,22 +294,34 @@ public class UploadPack {
 		return walk;
 	}
 
-	/** @return all refs which were advertised to the client. */
+	/**
+	 * Get refs which were advertised to the client.
+	 *
+	 * @return all refs which were advertised to the client, or null if
+	 *         {@link #setAdvertisedRefs(Map)} has not been called yet.
+	 */
 	public final Map<String, Ref> getAdvertisedRefs() {
-		if (refs == null)
-			setAdvertisedRefs(db.getAllRefs());
 		return refs;
 	}
 
 	/**
+	 * Set the refs advertised by this UploadPack.
+	 * <p>
+	 * Intended to be called from a {@link PreUploadHook}.
+	 *
 	 * @param allRefs
 	 *            explicit set of references to claim as advertised by this
 	 *            UploadPack instance. This overrides any references that
 	 *            may exist in the source repository. The map is passed
-	 *            to the configured {@link #getRefFilter()}.
+	 *            to the configured {@link #getRefFilter()}. If null, assumes
+	 *            all refs were advertised.
 	 */
 	public void setAdvertisedRefs(Map<String, Ref> allRefs) {
-		refs = refFilter.filter(allRefs);
+		if (allRefs != null)
+			refs = allRefs;
+		else
+			refs = db.getAllRefs();
+		refs = refFilter.filter(refs);
 	}
 
 	/** @return timeout (in seconds) before aborting an IO operation. */
@@ -330,18 +382,39 @@ public class UploadPack {
 		requestPolicy = policy != null ? policy : RequestPolicy.ADVERTISED;
 	}
 
+	/** @return the hook used while advertising the refs to the client */
+	public AdvertiseRefsHook getAdvertiseRefsHook() {
+		return advertiseRefsHook;
+	}
+
 	/** @return the filter used while advertising the refs to the client */
 	public RefFilter getRefFilter() {
 		return refFilter;
 	}
 
+	/**
+	 * Set the hook used while advertising the refs to the client.
+	 * <p>
+	 * If the {@link AdvertiseRefsHook} chooses to call
+	 * {@link #setAdvertisedRefs(Map)}, only refs set by this hook <em>and</em>
+	 * selected by the {@link RefFilter} will be shown to the client.
+	 *
+	 * @param advertiseRefsHook
+	 *            the hook; may be null to show all refs.
+	 */
+	public void setAdvertiseRefsHook(final AdvertiseRefsHook advertiseRefsHook) {
+		if (advertiseRefsHook != null)
+			this.advertiseRefsHook = advertiseRefsHook;
+		else
+			this.advertiseRefsHook = AdvertiseRefsHook.DEFAULT;
+	}
+
 	/**
 	 * Set the filter used while advertising the refs to the client.
 	 * <p>
-	 * Only refs allowed by this filter will be sent to the client. This can
-	 * be used by a server to restrict the list of references the client can
-	 * obtain through clone or fetch, effectively limiting the access to only
-	 * certain refs.
+	 * Only refs allowed by this filter will be sent to the client.
+	 * The filter is run against the refs specified by the
+	 * {@link AdvertiseRefsHook} (if applicable).
 	 *
 	 * @param refFilter
 	 *            the filter; may be null to show all refs.
@@ -391,6 +464,24 @@ public class UploadPack {
 		this.logger = logger;
 	}
 
+	/**
+	 * Check whether the client expects a side-band stream.
+	 *
+	 * @return true if the client has advertised a side-band capability, false
+	 *     otherwise.
+	 * @throws RequestNotYetReadException
+	 *             if the client's request has not yet been read from the wire, so
+	 *             we do not know if they expect side-band. Note that the client
+	 *             may have already written the request, it just has not been
+	 *             read.
+	 */
+	public boolean isSideBand() throws RequestNotYetReadException {
+		if (options == null)
+			throw new RequestNotYetReadException();
+		return (options.contains(OPTION_SIDE_BAND)
+				|| options.contains(OPTION_SIDE_BAND_64K));
+	}
+
 	/**
 	 * Execute the upload task on the socket.
 	 *
@@ -451,6 +542,12 @@ public class UploadPack {
 		return statistics;
 	}
 
+	private Map<String, Ref> getAdvertisedOrDefaultRefs() {
+		if (refs == null)
+			setAdvertisedRefs(null);
+		return refs;
+	}
+
 	private void service() throws IOException {
 		if (biDirectionalPipe)
 			sendAdvertisedRefs(new PacketLineOutRefAdvertiser(pckOut));
@@ -458,7 +555,7 @@ public class UploadPack {
 			advertised = Collections.emptySet();
 		else {
 			advertised = new HashSet<ObjectId>();
-			for (Ref ref : getAdvertisedRefs().values()) {
+			for (Ref ref : getAdvertisedOrDefaultRefs().values()) {
 				if (ref.getObjectId() != null)
 					advertised.add(ref.getObjectId());
 			}
@@ -488,7 +585,7 @@ public class UploadPack {
 			reportErrorDuringNegotiate(err.getMessage());
 			throw err;
 
-		} catch (UploadPackMayNotContinueException err) {
+		} catch (ServiceMayNotContinueException err) {
 			if (!err.isOutput() && err.getMessage() != null) {
 				try {
 					pckOut.writeString("ERR " + err.getMessage() + "\n");
@@ -562,14 +659,14 @@ public class UploadPack {
 	 *            the advertisement formatter.
 	 * @throws IOException
 	 *             the formatter failed to write an advertisement.
-	 * @throws UploadPackMayNotContinueException
+	 * @throws ServiceMayNotContinueException
 	 *             the hook denied advertisement.
 	 */
 	public void sendAdvertisedRefs(final RefAdvertiser adv) throws IOException,
-			UploadPackMayNotContinueException {
+			ServiceMayNotContinueException {
 		try {
-			preUploadHook.onPreAdvertiseRefs(this);
-		} catch (UploadPackMayNotContinueException fail) {
+			advertiseRefsHook.advertiseRefs(this);
+		} catch (ServiceMayNotContinueException fail) {
 			if (fail.getMessage() != null) {
 				adv.writeOne("ERR " + fail.getMessage());
 				fail.setOutput();
@@ -590,7 +687,7 @@ public class UploadPack {
 		if (!biDirectionalPipe)
 			adv.advertiseCapability(OPTION_NO_DONE);
 		adv.setDerefTags(true);
-		advertised = adv.send(getAdvertisedRefs());
+		advertised = adv.send(getAdvertisedOrDefaultRefs());
 		adv.end();
 	}
 
@@ -623,12 +720,9 @@ public class UploadPack {
 				throw new PackProtocolException(MessageFormat.format(JGitText.get().expectedGot, "want", line));
 
 			if (isFirst && line.length() > 45) {
-				String opt = line.substring(45);
-				if (opt.startsWith(" "))
-					opt = opt.substring(1);
-				for (String c : opt.split(" "))
-					options.add(c);
-				line = line.substring(0, 45);
+				final FirstLine firstLine = new FirstLine(line);
+				options = firstLine.getOptions();
+				line = firstLine.getLine();
 			}
 
 			wantIds.add(ObjectId.fromString(line.substring(5)));
@@ -959,7 +1053,7 @@ public class UploadPack {
 		if (sideband) {
 			try {
 				sendPack(true);
-			} catch (UploadPackMayNotContinueException noPack) {
+			} catch (ServiceMayNotContinueException noPack) {
 				// This was already reported on (below).
 				throw noPack;
 			} catch (IOException err) {
@@ -1023,7 +1117,7 @@ public class UploadPack {
 			} else {
 				preUploadHook.onSendPack(this, wantAll, commonBase);
 			}
-		} catch (UploadPackMayNotContinueException noPack) {
+		} catch (ServiceMayNotContinueException noPack) {
 			if (sideband && noPack.getMessage() != null) {
 				noPack.setOutput();
 				SideBandOutputStream err = new SideBandOutputStream(
